import sys

from composipy.pre_integrated_component.functions import *


def calc_K11_ijkl(a, b, ui, uj, uk, ul, A11, A16, A66):
    uik = (ui, uk)
    ujl = (uj, ul)
    uxi_ij_uxi_kl = ii_fxi_fxi(uik) * ii_ff(ujl)
    uxi_ij_ueta_kl = ii_fxi_f(uik) * ii_f_fxi(ujl)
    ueta_ij_uxi_kl = ii_f_fxi(uik) * ii_fxi_f(ujl)
    ueta_ij_ueta_kl = ii_ff(uik) * ii_fxi_fxi(ujl)

    k11 = (
        A11 * (b/a) *  uxi_ij_uxi_kl
        + A16 * (uxi_ij_ueta_kl+ueta_ij_uxi_kl)
        + A66 * (b/a) * ueta_ij_ueta_kl)
    return k11


def calc_k12_ijkl(a, b, ui, uj, vk, vl, A12, A16, A26, A66):
    uivk = (ui, vk)
    ujvl = (uj, vl)
    uxi_ij_veta_kl = ii_fxi_f(uivk) * ii_f_fxi(ujvl)
    uxi_ij_vxi_kl = ii_fxi_fxi(uivk) * ii_ff(ujvl)
    ueta_ij_veta_kl = ii_ff(uivk) * ii_fxi_fxi(ujvl)
    ueta_ij_vxi_kl = ii_f_fxi(uivk) * ii_fxi_f(ujvl)

    k12 = (
        A12 * uxi_ij_veta_kl
        + A16 * (b/a) * uxi_ij_vxi_kl
        + A26 * (a/b) * ueta_ij_veta_kl
        + A66 * ueta_ij_vxi_kl
    )
    return k12


def calc_k13_ijkl(a, b, ui, uj, wk, wl, B11, B12, B16, B26, B66):
    uiwk = (ui, wk)
    ujwl = (uj, wl)
    uxi_ij_wxixi_kl = ii_fxi_fxixi(uiwk) * ii_ff(ujwl)
    uxi_wetaeta = ii_fxi_f(uiwk) * ii_f_fxixi(ujwl)
    uxi_wxieta = ii_fxi_fxi(uiwk) * ii_f_fxi(ujwl)
    ueta_wxixi = ii_f_fxixi(uiwk) * ii_fxi_f(ujwl)
    ueta_wetaeta = ii_ff(uiwk) * ii_fxi_fxixi(ujwl)
    ueta_wxieta = ii_f_fxi(uiwk) * ii_fxi_fxi(ujwl)
    uxi_w = ii_fxi_f(uiwk) * ii_ff(ujwl)
    ueta_w = ii_ff(uiwk) * ii_fxi_f(ujwl)

    k13 = (
        -(2*b/a) * B11 * uxi_ij_wxixi_kl
        -(2/b) * B12 * uxi_wetaeta
        -(2/a) * B16 * (2*uxi_wxieta + ueta_wxixi)
        -(2/b**2) * B26 * ueta_wetaeta
        -(4/b) * B66 * ueta_wxieta
    )

    return k13


def calc_k21_ijkl(a, b, vi, vj, uk, ul, A12, A16, A26, A66):
    viuk = (vi, uk)
    vjul = (vj, ul)
    veta_ij_uxi_kl = ii_f_fxi(viuk) * ii_fxi_f(vjul)
    vxi_ij_uxi_kl = ii_fxi_fxi(viuk) * ii_ff(vjul)
    veta_ij_ueta_ij = ii_ff(viuk) * ii_fxi_fxi(vjul)
    vxi_ij_ueta_kl = ii_fxi_f(viuk) * ii_f_fxi(vjul)

    k21 = (
        A12 * veta_ij_uxi_kl
        + A16 * (a/b) * vxi_ij_uxi_kl
        + A26 * (a/b) * veta_ij_ueta_ij
        + A66 * vxi_ij_ueta_kl
    )
    return k21


def calc_k22_ijkl(a, b, vi, vj, vk, vl, A22, A26, A66):
    vivk = (vi, vk)
    vjvl = (vj, vl)
    veta_ij_veta_kl = ii_ff(vivk) * ii_fxi_fxi(vjvl)
    vxi_ij_veta_kl = ii_fxi_f(vivk) * ii_f_fxi(vjvl)
    veta_ij_vxi_kl = ii_f_fxi(vivk) * ii_fxi_f(vjvl)
    vxi_ij_vxi_kl = ii_fxi_fxi(vivk) * ii_ff(vjvl)

    k22 = (
        A22 * (a/b) * veta_ij_veta_kl
        + A26 * (vxi_ij_veta_kl+veta_ij_vxi_kl)
        + A66 * (b/a) * vxi_ij_vxi_kl
    )
    return k22


def calc_k23_ijkl(a, b, vi, vj, wk, wl, B12, B16, B22, B26, B66):
    viwk = (vi, wk)
    vjwl = (vj, wl)
    veta_wxixi = ii_f_fxixi(viwk) * ii_fxi_f(vjwl)
    vxi_wxixi = ii_fxi_fxixi(viwk) * ii_ff(vjwl)
    veta_wetaeta = ii_ff(viwk) * ii_fxi_fxixi(vjwl)
    vxi_wetaeta = ii_fxi_f(viwk) * ii_f_fxixi(vjwl)
    veta_wxieta = ii_f_fxi(viwk) * ii_fxi_fxi(vjwl)
    vxi_wxieta = ii_fxi_fxi(viwk) * ii_f_fxi(vjwl)

    k23 = (
        -(2/a) * B12 * veta_wxixi
        -(2*b/(a**2)) * B16 * vxi_wxixi
        -(2*a/(b**2)) * B22 * veta_wetaeta
        -(2/b) * B26 * (vxi_wetaeta + 2*veta_wxieta)
        -(4/a) * B66 * (vxi_wxieta)
    )

    return k23


def calc_k31_ijkl(a, b, wi, wj, uk, ul, B11, B12, B16, B26, B66):
    wiuk = (wi, uk)
    wjul = (wj, ul)
    wxixi_uxi = ii_fxi_fxixi(wiuk) * ii_ff(wjul)
    wetaeta_uxi = ii_f_fxi(wiuk) * ii_fxixi_f(wjul)
    wetaeta_ueta = ii_ff(wiuk) * ii_fxixi_fxi(wjul)
    wxieta_ueta = ii_fxi_f(wiuk) * ii_fxi_fxi(wjul)
    wxieta_uxi = ii_fxi_fxi(wiuk) * ii_fxi_f(wjul)
    wxixi_ueta = ii_fxixi_f(wiuk) * ii_f_fxi(wjul)
    
    k31 = (
        -(2*b/a**2) * B11 * wxixi_uxi
        -(2/b) * B12 * wetaeta_uxi
        -(2*a/b**2) * B26 * wetaeta_ueta
        -(4/b) * B66 * wxieta_ueta
        -(2/a) * B16 * (2*wxieta_uxi+wxixi_ueta)
    )

    return k31


def calc_k32_ijkl(a, b, wi, wj, vk, vl, B11, B12, B16, B22, B26, B66):
    wivk = (wi, vk)
    wjvl = (wj, vl)
    wxixi_veta = ii_fxixi_f(wivk) * ii_f_fxi(wjvl)
    wetaeta_veta = ii_ff(wivk) * ii_fxixi_fxi(wjvl)
    wxieta_vxi = ii_fxi_fxi(wivk) * ii_fxi_f(wjvl)
    wetaeta_vxi = ii_f_fxi(wivk) * ii_fxixi_f(wjvl)
    wxieta_veta = ii_fxi_f(wivk) * ii_fxi_fxi(wjvl)
    wxixi_vxi = ii_fxixi_fxi(wivk) * ii_ff(wjvl)

    k32 = (
        -(2/a) * B12 * wxixi_veta
        -(2*a/(b**2)) * B22 * wetaeta_veta
        -(4/a) * B66 * wxieta_vxi
        -(2/b) * B26 * (wetaeta_vxi+2*wxieta_veta)
        -(2*b/a**2) * B16 * wxixi_vxi
    )

    return k32


def calc_k33_ijkl(a, b, wi, wj, wk, wl, D11, D12, D22, D16, D26, D66):
    wiwk = (wi, wk)
    wjwl = (wj, wl)
    wxixi_ij_wxixi_kl = ii_fxixi_fxixi(wiwk) * ii_ff(wjwl)
    wxixi_ij_wetaeta_kl = ii_fxixi_f(wiwk) * ii_f_fxixi(wjwl)
    wetaeta_ij_wxixi_ij = ii_f_fxixi(wiwk) * ii_fxixi_f(wjwl)
    wetaeta_ij_wetaeta_kl = ii_ff(wiwk) * ii_fxixi_fxixi(wjwl)
    wxixi_ij_wxieta_kl = ii_fxixi_fxi(wiwk) * ii_f_fxi(wjwl)
    wxieta_ij_wxixi_kl = ii_fxi_fxixi(wiwk) * ii_fxi_f(wjwl)
    wxieta_ij_wetaeta_kl = ii_fxi_f(wiwk) * ii_fxi_fxixi(wjwl)
    wetaeta_ij_wxieta_kl = ii_f_fxi(wiwk) * ii_fxixi_fxi(wjwl)
    wxieta_ij_wxieta_kl = ii_fxi_fxi(wiwk) * ii_fxi_fxi(wjwl)

    k33 = (
        4*(b/a**3) * D11 * wxixi_ij_wxixi_kl
        + (4/(a*b)) * D12 * (wxixi_ij_wetaeta_kl+wetaeta_ij_wxixi_ij)
        + 4*(a/b**3) * D22 * wetaeta_ij_wetaeta_kl
        + 8/a**2 * D16 * (wxixi_ij_wxieta_kl+wxieta_ij_wxixi_kl)
        + 8/b**2 * D26 * (wxieta_ij_wetaeta_kl+wetaeta_ij_wxieta_kl)
        + (16/(a*b)) * D66 * wxieta_ij_wxieta_kl
    )
    return k33


def calc_kG33_ijkl(a, b, wi, wj, wk, wl, Nxx, Nyy, Nxy):
    wiwk = (wi, wk)
    wjwl = (wj, wl)
    wxi_ij_wxi_kl = ii_fxi_fxi(wiwk) * ii_ff(wjwl)
    weta_ij_weta_kl = ii_ff(wiwk) * ii_fxi_fxi(wjwl)
    weta_ij_wxi_kl = ii_f_fxi(wiwk) * ii_fxi_f(wjwl)
    wxi_ij_weta_kl = ii_fxi_f(wiwk) * ii_f_fxi(wjwl)

    kG33 = (
        Nxx * (b/a) * wxi_ij_wxi_kl
        + Nyy * (a/b) * weta_ij_weta_kl
        + Nxy * (weta_ij_wxi_kl+wxi_ij_weta_kl)
    )
    return kG33



import numpy as np

from time import time
from scipy.sparse import csr_matrix
from scipy.sparse.linalg import eigsh
from itertools import product

from composipy.core.property import Property
from composipy.utils import ComposipyValidator
from composipy.pre_integrated_component.build_k import *


class Structure(ComposipyValidator):
      pass


class PlateStructure(Structure):
    '''
    This class defines a PlateStructure.

    Parameters
    ----------
    dproperty : composipy.Property
        Property of the plate.
    a : float
        Size of the plate parallel to the x axis.
    b : float
        Size of the plate parallel to the y axis.
    constraints : str or dict. Default: \"PINNED\"
        Plate boundary conditions.
    Nxx : float, default 0
        Linear force parallel x axis 
    Nyy : float, default 0
        Linear force parallel y axis
    Nxy : float, default 0
        Linear shear force
    m : int, default 10
        Size of shape function along x axis
    n : int, default 10
        Size of shape function along y axis


    Example
    -------
    >>> from composipy import OrthotropicMaterial, LaminateProperty, PlateStructure
    >>> ply_1 = OrthotropicMaterial(129500, 9370, 0.38, 5240, 0.2)
    >>> stacking = [90, 0, 90]
    >>> laminate = Laminate(stacking, ply_1)
    >>> constraints

    Note
    -----
    The ``constraint`` argument can be a str type \'PINNED\' or \'CLAMPED\'.
    Or a dictionary like described below:

    >>> constraints = {    
    ---     x0 = ['TX', 'TY', 'TZ', 'RX', 'RY', 'RZ']
    ---     xa = ['TX', 'TY', 'TZ', 'RX', 'RY', 'RZ']
    ---     y0 = ['TX', 'TY', 'TZ', 'RX', 'RY', 'RZ']
    ---     yb = ['TX', 'TY', 'TZ', 'RX', 'RY', 'RZ']
    --- }

    Attention, the rotations aren't considered around the axis. They are related to the shape function.
    That means, for example, the ``RZ`` turns around the x axis and moves the plate in z direction. 
    '''



    def __init__(
            self,
            dproperty,
            a,
            b,
            constraints={
                'x0': ['TX', 'TY', 'TZ'],
                'xa': ['TX', 'TY', 'TZ'],
                'y0': ['TX', 'TY', 'TZ'],
                'yb': ['TX', 'TY', 'TZ'],
            },
            Nxx=0,
            Nyy=0,
            Nxy=0,
            m=10,
            n=10):

        self.__dproperty = self._is_instance(dproperty, Property, 'dproperty')
        self.__a = self._float_number(a, n_min=0, name='a')
        self.__b = self._float_number(b, n_min=0, name='b')
        self.__constraints = constraints
        self.__Nxx = self._float_number(Nxx, name='Nxx')
        self.__Nyy = self._float_number(Nyy, name='Nyy')
        self.__Nxy = self._float_number(Nxy, name='Nxy')
        self.__m = self._int_number(m, n_min=1, name='m')
        self.__n = self._int_number(n, n_min=1, name='n')

    @property
    def dproperty(self):
            return self.__dproperty
        
    @property
    def a(self):
        return self.__a
    
    @property
    def b(self):
            return self.__b

    @property
    def constraints(self):
            return self.__constraints
    
    @property
    def Nxx(self):
            return self.__Nxx
    @property
    def Nyy(self):
            return self.__Nyy
    
    @property
    def Nxy(self):
            return self.__Nxy
    
    @property
    def m(self):
            return self.__m
    
    @property
    def n(self):
            return self.__n


    def _compute_constraints(self):
        if self.constraints == 'PINNED':
            x0 = ['TX', 'TY', 'TZ']
            xa = ['TX', 'TY', 'TZ']
            y0 = ['TX', 'TY', 'TZ']
            yb = ['TX', 'TY', 'TZ']
        elif self.constraints == 'CLAMPED':
            x0 = ['TX', 'TY', 'TZ', 'RX', 'RY', 'RZ']
            xa = ['TX', 'TY', 'TZ', 'RX', 'RY', 'RZ']
            y0 = ['TX', 'TY', 'TZ', 'RX', 'RY', 'RZ']
            yb = ['TX', 'TY', 'TZ', 'RX', 'RY', 'RZ']
        else:
            x0 = self.constraints['x0']
            xa = self.constraints['xa']
            y0 = self.constraints['y0']
            yb = self.constraints['yb']

        sm = [i for i in range(self.m+4)]
        sn = [i for i in range(self.n+4)]

        um, un = sm.copy(), sn.copy()
        vm, vn = sm.copy(), sn.copy()
        wm, wn = sm.copy(), sn.copy()

        # x0
        if 'TX' in x0:
              um.remove(0)
        if 'TY' in x0:
              vm.remove(0)
        if 'TZ' in x0:
              wm.remove(0)
        if 'RX' in x0:
              um.remove(1)
        if 'RY' in x0:
              vm.remove(1)
        if 'RZ' in x0:
              wm.remove(1)
        #xa
        if 'TX' in xa:
              um.remove(2)
        if 'TY' in xa:
              vm.remove(2)
        if 'TZ' in xa:
              wm.remove(2)
        if 'RX' in xa:
              um.remove(3)
        if 'RY' in xa:
              vm.remove(3)
        if 'RZ' in xa:
              wm.remove(3)
        
        #y0
        if 'TX' in y0:
              un.remove(0)
        if 'TY' in y0:
              vn.remove(0)
        if 'TZ' in y0:
              wn.remove(0)
        if 'RX' in y0:
              un.remove(1)
        if 'RY' in y0:
              vn.remove(1)
        if 'RZ' in y0:
              wn.remove(1)
        
        #yb
        if 'TX' in yb:
              un.remove(2)
        if 'TY' in yb:
              vn.remove(2)
        if 'TZ' in yb:
              wn.remove(2)
        if 'RX' in yb:
              un.remove(3)
        if 'RY' in yb:
              vn.remove(3)
        if 'RZ' in yb:
              wn.remove(3)
        
        um, un = um[0:self.m], un[0:self.n]
        vm, vn = vm[0:self.m], un[0:self.n]
        wm, wn = wm[0:self.m], un[0:self.n]

        uidx = list(product(um, un, um, un))
        vidx = list(product(vm, vn, vm, vn))
        widx = list(product(wm, wn, wm, wn))

        return (uidx, vidx, widx)
    
    #TODO: parallel process, sparse matrix, cython
    def calc_K_KG(self):
        '''
        Calculates the stiffness and geometrical stifness matrices.

        Returns
        -------
        K_KG : tuple
            A tuple of array containing (K, KG)
        '''

        A11, A12, A16, B11, B12, B16 = self.dproperty.ABD[0, ::]
        A12, A22, A26, B12, B22, B26 = self.dproperty.ABD[1, ::]
        A16, A26, A66, B16, B26, B66 = self.dproperty.ABD[2, ::]
        B11, B12, B16, D11, D12, D16 = self.dproperty.ABD[3, ::]
        B12, B22, B26, D12, D22, D26 = self.dproperty.ABD[4, ::]
        B16, B26, B66, D16, D26, D66 = self.dproperty.ABD[5, ::]

        k11, k12, k13, k21, k22, k23, k31, k32, k33 = [], [], [], [], [], [], [], [], []
        k33g = []

        uidx, vidx, widx = self._compute_constraints()

        for i in range(self.m**2*self.n**2):
                ui, uj, uk, ul = uidx[i]
                vi, vj, vk, vl = vidx[i]
                wi, wj, wk, wl = widx[i]

                k11.append(calc_K11_ijkl(self.a, self.b, ui, uj, uk, ul, A11, A16, A66))
                k12.append(calc_k12_ijkl(self.a, self.b, ui, uj, vk, vl, A12, A16, A26, A66))
                k13.append(calc_k13_ijkl(self.a, self.b, ui, uj, wk, wl, B11, B12, B16, B26, B66))
                k21.append(calc_k21_ijkl(self.a, self.b, vi, vj, uk, ul, A12, A16, A26, A66))
                k22.append(calc_k22_ijkl(self.a, self.b, vi, vj, vk, vl, A22, A26, A66))
                k23.append(calc_k23_ijkl(self.a, self.b, vi, vj, wk, wl, B12, B16, B22, B26, B66))
                k31.append(calc_k31_ijkl(self.a, self.b, wi, wj, uk, ul, B11, B12, B16, B26, B66))
                k32.append(calc_k32_ijkl(self.a, self.b, wi, wj, vk, vl, B11, B12, B16, B22, B26, B66))
                k33.append(calc_k33_ijkl(self.a, self.b, wi, wj, wk, wl, D11, D12, D22, D16, D26, D66))
                k33g.append(calc_kG33_ijkl(self.a, self.b, wi, wj, wk, wl, self.Nxx, self.Nyy, self.Nxy))
        
        k11 = np.array(k11).reshape(self.m*self.n, self.m*self.n)
        k12 = np.array(k12).reshape(self.m*self.n, self.m*self.n)
        k13 = np.array(k13).reshape(self.m*self.n, self.m*self.n)
        k21 = np.array(k21).reshape(self.m*self.n, self.m*self.n)
        k22 = np.array(k22).reshape(self.m*self.n, self.m*self.n)
        k23 = np.array(k23).reshape(self.m*self.n, self.m*self.n)
        k31 = np.array(k31).reshape(self.m*self.n, self.m*self.n)
        k32 = np.array(k32).reshape(self.m*self.n, self.m*self.n)
        k33 = np.array(k33).reshape(self.m*self.n, self.m*self.n)
        k00 = np.zeros(self.m**2*self.n**2).reshape(self.m*self.n, self.m*self.n)
        k33g = np.array(k33g).reshape(self.m*self.n, self.m*self.n)

        K = np.vstack([
        np.hstack([k11, k12, k13]),
        np.hstack([k21, k22, k31]),
        np.hstack([k31, k32, k33])
        ])

        KG = np.vstack([
        np.hstack([k00, k00, k00]),
        np.hstack([k00, k00, k00]),
        np.hstack([k00, k00, k33g])
        ])

        return K, KG  

    def buckling_analysis(self, silent=True, num_eigvalues = 5):
        """
        Linear Buckling Analysis

        Parameters
        ----------
        silent : bool, optional
            Prints time to compute calculation
        num_eigvalues : int, optional
            Number of calculated eigenvalues.

        Returns
        -------
        eigvals, eigvecs
        """
        if not silent:
            ti = time()       
            print('calculating K and KG')
        K, KG = self.calc_K_KG()
        K, KG = csr_matrix(K), csr_matrix(KG)
        if not silent:
            print(f'K and KG calculated in {time()-ti} seconds')
        tol = 0
    
        if not silent:
            ti = time()
            print('Calculating eigenvalues')
        k = min(num_eigvalues, KG.shape[0]-2)
        mode = 'cayley'

        eigvals, eigvecs = eigsh(A=KG, k=k,
              which='SM', M=K, tol=tol, sigma=1., mode=mode)
        
        eigvals = -1./eigvals
    
        eigvals = eigvals
        eigvecs = eigvecs 
        if not silent:
             print(f'eigenvalues calculated in {time()-ti} seconds')

        return eigvals, eigvecs


import sympy as sp
import numpy as np
from time import time

ti = time()

xi, eta = sp.symbols(['xi', 'eta'])
f1 = 1/2 - 3/4*xi + 1/4*xi**3
f2 = 1/8 - 1/8*xi - 1/8*xi**2 + 1/8*xi**3
f3 = 1/2 + 3/4*xi - 1/4*xi**3
f4 = -1/8 - 1/8*xi + 1/8*xi**2 + 1/8*xi**3

F = {0: f1, 1: f2, 2: f3, 3: f4} # first terms of set of functions
FXI = {
    0: sp.diff(f1, xi),
    1: sp.diff(f2, xi),
    2: sp.diff(f3, xi),
    3: sp.diff(f4, xi)   
}
FXIXI = {
    0: sp.diff(f1, xi, xi),
    1: sp.diff(f2, xi, xi),
    2: sp.diff(f3, xi, xi),
    3: sp.diff(f4, xi, xi)   
}


def _orth_polynomials(r, sym):   
    '''
    r term from the set of functions used by Bardell (1991)

    Parameters
    ----------
    r : int
    sym : sympy.symbol
        symbol to be used in equation

    Returns
    -------
    f_sym_r : sympy equation
    r term from the set of functions.
    '''
    nmax = r//2 + 1 #denotes its own integer part (Bardel, 1991).
    f_sym_r = 0

    for n in range(0, nmax):
        f_sym_r += (
            (-1)**n * sp.factorial2(2*r-2*n-7)
            / (2**n * sp.factorial(n) * sp.factorial(r-2*n-1))
            * sym**(r-2*n-1)
            )
    f_sym_r = sp.sympify(f_sym_r)
    return f_sym_r


def convert_to_str(d, name):
    '''
    Parameters
    ----------
    d : dict 
        dictionary to be converted into string
    name : str
        name of the variable
    '''

    txt = name + ' = {\n'
    for k, v in d.items():
        txt += '    ' + str(k) + ': \'' + str(v) + '\',\n'
    txt += '}' + '\n' * 3
    return txt


if __name__ == '__main__':
    
    n_terms = 50

    # Complete set of bardell functions
    for i in range(4, n_terms):
        F[i] = _orth_polynomials(i+1, xi)
        FXI[i] = sp.diff(F[i], xi)
        FXIXI[i] = sp.diff(F[i], xi, xi)


    FF, FXI_F, FXI_FXI, FXIXI_F, FXIXI_FXI, FXIXI_FXIXI = {}, {}, {}, {}, {}, {} # shape function
    ii_FF, ii_FXI_F, ii_FXI_FXI, ii_FXIXI_F, ii_FXIXI_FXI, ii_FXIXI_FXIXI = {}, {}, {}, {}, {}, {} # shape function

    for i in range(n_terms):
        for k in range(n_terms):
            FF[(i, k)] = F[i] * F[k]
            FXI_F[(i, k)] = FXI[i] * F[k]
            FXI_FXI[(i, k)] = FXI[i] * FXI[k]
            FXIXI_F[(i, k)] = FXIXI[i] * F[k]
            FXIXI_FXI[(i, k)] = FXIXI[i] * FXI[k]
            FXIXI_FXIXI[(i, k)] = FXIXI[i] * FXIXI[k]

            ii_FF[(i, k)] = sp.integrate(FF[(i, k)], (xi, -1, 1))
            ii_FXI_F[(i, k)] = sp.integrate(FXI_F[(i, k)], (xi, -1, 1))
            ii_FXI_FXI[(i, k)] = sp.integrate(FXI_FXI[(i, k)], (xi, -1, 1))
            ii_FXIXI_F[(i, k)] = sp.integrate(FXIXI_F[(i, k)], (xi, -1, 1))
            ii_FXIXI_FXI[(i, k)] = sp.integrate(FXIXI_FXI[(i, k)], (xi, -1, 1))
            ii_FXIXI_FXIXI[(i, k)] = sp.integrate(FXIXI_FXIXI[(i, k)], (xi, -1, 1))
     

    txt = f'# This file is generated by {__file__}, it contains the preintegrated terms\n\n'
    txt += '__all__ = [\'ii_FF\', \'ii_FXI_F\', \'ii_FXI_FXI\', \'ii_FXIXI_F\', \'ii_FXIXI_FXI\', \'ii_FXIXI_FXIXI\']\n\n\n'
    txt += convert_to_str(ii_FF, 'ii_FF')
    txt += convert_to_str(ii_FXI_F, 'ii_FXI_F')
    txt += convert_to_str(ii_FXI_FXI, 'ii_FXI_FXI')
    txt += convert_to_str(ii_FXIXI_F, 'ii_FXIXI_F')
    txt += convert_to_str(ii_FXIXI_FXI, 'ii_FXIXI_FXI')
    txt += convert_to_str(ii_FXIXI_FXIXI, 'ii_FXIXI_FXIXI')


    with open('_ii_F.py', 'w') as f:
        f.write(txt)
        
    print(f'time is {time() - ti} s')





__all__ = ['build_sequence', 'build_pcomp']

def _build_pm_unit(angle, times):
    '''Returns a list when there is the plus minus sign '''
    stacking_values = list()
    
    for i in range(times):
        current_angle = (-1)**i*angle
        stacking_values += [current_angle]
    return stacking_values


def _build_stack_unit(stacking):
    '''Returns a list when given a unit for example \\pm45_13'''
    stacking_values = stacking
    is_pm = stacking_values.find('\pm') > -1
    stacking_values = stacking_values.replace('\pm', '')
    stacking_values = stacking_values.strip()   
    stacking_values = stacking_values.split('_')
    len_stacking_values = len(stacking_values)
    is_unit = (len_stacking_values == 2
              or len_stacking_values == 1)

    if not is_unit:
        raise ValueError(f'{stacking} it is not a unit')
    angle = float(stacking_values[0])
    if len_stacking_values == 2:
        times = stacking_values[1].replace('{', '').replace('}', '')
        times = int(times)
    elif len_stacking_values == 1:
        times = 1
    
    if is_pm:
        return _build_pm_unit(angle, times)
    else:
        return [angle] * times
   
    
def build_sequence(stacking):
    '''
    Parameters
    ----------
    stacking : str
        A string in Latex format
    
    Returns
    -------
    unities : list
        A list that contains the stacking sequence
        
    Example
    -------
    >>> stack = '[\pm45_{1}/0_2/90/(0/90)_1]s'
    >>> build_sequence(stack)
    out : 
        [45.0, -45.0, 0.0, 90.0, 0.0, 90.0, 90.0, 0.0, 90.0, 0.0, -45.0, 45.0]
    '''
    stacking = stacking.strip()
    is_symmetric = stacking[-1].lower() == 's'
    stacking = stacking.replace('s', '')
    is_stacking = (stacking[0] == '['
                   and stacking[-1] == ']')
    stacking = stacking.replace('[', '').replace(']', '')
    stacking = stacking + '/'
    
    unities = list()
    unity = list()
    block = list()
    current_block = ''
    is_in_block = False
    is_multiplier = False
    
    for i, char in enumerate(stacking):
        if char == '(':
            is_in_block = True
        elif char == ')':
            unity = _build_stack_unit(current_block)
            block.extend(unity)
            current_block = ''
            is_multiplier = True
        elif (char == '/' 
              and not is_in_block
              and not is_multiplier
             ):
            unity = _build_stack_unit(current_block)
            unities.extend(unity)
            current_block = ''
            unity = list()
        elif (char == '/' 
              and is_in_block
              and not is_multiplier):
            unity = _build_stack_unit(current_block)
            block.extend(unity)
            current_block = ''
        elif (char == '/' 
             and is_multiplier
             and not is_in_block):
            times = current_block.replace('_', '').replace('{', '').replace('}', '')
            times = int(times)
            unity = times * unity
            unities.extend(unity)
            unity = list()
            current_block = ''
            is_multiplier = False
        elif (char == '/'
              and is_multiplier
              and is_in_block):
            times = current_block.replace('_', '').replace('{', '').replace('}', '')
            times = int(times)
            unity = times * block
            unities.extend(unity)
            unity = list()
            block = list()
            current_block = ''
            is_multiplier = False
            is_in_block = False
        else:
            current_block += char
           
    if is_symmetric:
        symmetric_unities = unities[::-1]
        unities.extend(symmetric_unities)
    return unities


def _convert_to_list(value, size):
    '''If it is list return value, if value is a number return a list'''
    try:
        size_value = len(value)
        if size_value == size:
            return value
        else:
            raise ValueError(f'{value} does not fit with sequence size')
    except TypeError:
        return [value for i in range(size)]

def _convert_sout(sout, size):
    if sout == 'FIBER':
        sout = ['NO' for i in range(size)]
        sout[0] = 'YES'
        sout[-1] = 'YES'
        return sout
    elif sout == 'NO':
        sout = ['NO' for i in range(size)]
        return sout
    elif sout == 'YES':
        sout = ['YES' for i in range(size)]
        return sout
    else:
        try:
            size_value = len(sout)
            if size_value == size:
                return sout
            else:
                 raise ValueError
        except:
            raise ValueError(f'{sout} not accepted')
            

def build_pcomp(sequence, midi, ti, pid=1, z0='',sout='FIBER'):
    '''
    Parameters
    ----------
    sequence : list
        Angles of the stacking sequence. A list (or a iterable) containing angles
    midi : list or int
        A list of materials of plies or a material id (int) to be applied to all plies.
    ti : list or float
        A list of thickness of plies or a thickness (float) to be applied to all plies.
    pid : int, default 1
        NASTRAN property id
    z0 : float, default ''
        Laminate offset
    sout : list or str, default 'FIBER'
        A list of output request of plies or a output request to be applied to all plies.
        Options = YES, NO, FIBER
        If FIBER is used, then only the first and the last plies will be set as YES
    
    Returns
    -------
    text : str
        Returns a PCOMP card. 
    '''
    size = len(sequence)
    pid = int(pid)
    ti = _convert_to_list(ti, size)
    midi = _convert_to_list(midi, size)
    sout = _convert_sout(sout, size)
    
    header = f'PCOMP,{pid},{z0},,,,,+\n'
    body = ''
    
    for i in range(0, size, 2):
        i0 = i
        i1 = i + 1
        body += f'+,{midi[i0]},{ti[i0]},{sequence[i0]},{sout[i0]},'
        try:
            body += f'{midi[i1]},{ti[i1]},{sequence[i1]},{sout[i1]}+\n'
        except IndexError:
            body = body[0:-1] # removes last comma
    
    return header + body


import numbers

import numpy as np
import scipy as sp

from composipy.utils import ComposipyValidator
from composipy.core.material import Material, OrthotropicMaterial, IsotropicMaterial


class Property(ComposipyValidator):
    pass


class LaminateProperty(Property):
    '''
    This class creates laminate object. It needs Material objects (to define plies) and the angle information.
    Some formulation characteristics are:
    Laminate formulations ares used (see References)
    Main reference is the chapter 4 of reference 2.

    Parameters
    ----------
    stacking : list
        An iterable containing the angles (in degrees) of layup.
    plies : Material or list
        A single Material or a list of OrthotropicMaterial object


    Example
    -------
    >>> from composipy import OrthotropicMaterial, LaminateProperty
    >>> ply_1 = OrthotropicMaterial(129500, 9370, 0.38, 5240, 0.2)
    >>> stacking = [90, 0, 90]
    >>> laminate = Laminate(stacking, ply_1)
    >>> laminate_1.ABD # retunrs an array containing bending stiffness matrix [D] of the laminate
    >>> laminate_1.xiA # lamination parameters of extension
    >>> laminate_1.xiA # lamination parameters of bending

    '''

    def __init__(self, stacking, plies):
        # Checking layup
        if isinstance(plies, Material):
            n_plies = len(stacking)
            plies = [plies for s in range(n_plies)]
        elif len(plies) != len(stacking):
            raise ValueError('Number of plies and number of stacking must match')

        self.stacking = stacking
        self.plies = plies
        self.layup = list(zip(stacking, plies))
        self._z_position = None
        self._Q_layup = None
        self._T_layup = None
        self._A = None
        self._B = None
        self._D = None
        self._ABD = None
        self._ABD_p = None
        self._xiA = None
        self._xiB = None

#Properties
    @property
    def z_position(self):
    
        total_thickness = 0
        for t in self.layup:
            total_thickness += t[1].thickness
        
        current_z = -total_thickness/2
        ply_position = [current_z]
        for t in self.layup:
            current_z += t[1].thickness
            ply_position.append(current_z)
        
        return ply_position
    
    @property
    def Q_layup(self):    
        if self._Q_layup is None:
            
            self._Q_layup = []
            for theta, ply in self.layup:
                c = np.cos(theta*np.pi/180)
                s = np.sin(theta*np.pi/180)

                T_real = np.array([
                    [c**2, s**2, 2*c*s],
                    [s**2, c**2, -2*c*s],
                    [-c*s, c*s, c**2-s**2]
                    ])
                T_engineering =  np.array([
                    [c**2, s**2, c*s],
                    [s**2, c**2, -c*s],
                    [-2*c*s, 2*c*s, c**2-s**2]
                    ])

                self._Q_layup.append(
                    (np.linalg.inv(T_real))
                    @ ply.Q_0 
                    @ T_engineering
                    )
        return self._Q_layup
     
    @property
    def T_layup(self):
        if self._T_layup is None:
            
            self._T_layup = []
            for theta in self.layup:
                c = np.cos(theta[0]*np.pi/180)
                s = np.sin(theta[0]*np.pi/180)

                T_real = np.array([
                    [c**2, s**2, 2*c*s],
                    [s**2, c**2, -2*c*s],
                    [-c*s, c*s, c**2-s**2]
                    ])

                T_engineering =  np.array([
                    [c**2, s**2, c*s],
                    [s**2, c**2, -c*s],
                    [-2*c*s, 2*c*s, c**2-s**2]
                    ])

                self._T_layup.append([T_real,T_engineering])
        return self._T_layup

    @property
    def A(self):
        '''[A] Matrix as numpy.ndarray '''

        if self._A is None:
            self._A = np.zeros(9).reshape(3,3)

            for i in enumerate(self.Q_layup):
                zk1 = self.z_position[i[0]+1]
                zk0 = self.z_position[i[0]]
                self._A += (zk1-zk0) * i[1]
        return self._A
    
    @property
    def B(self):
        '''[B] Matrix as numpy.ndarray '''

        if self._B is None:
            self._B = np.zeros(9).reshape(3,3)

            for i in enumerate(self.Q_layup):
                zk1 = self.z_position[i[0]+1]
                zk0 = self.z_position[i[0]]
                self._B += (1/2) * (zk1**2-zk0**2) * i[1]
        return self._B
    
    @property
    def D(self):
        '''[D] Matrix as numpy.ndarray '''

        if self._D is None:
            self._D = np.zeros(9).reshape(3,3)

            for i in enumerate(self.Q_layup):
                zk1 = self.z_position[i[0]+1]
                zk0 = self.z_position[i[0]]
                self._D += (1/3) * (zk1**3-zk0**3) * i[1]
        return self._D

    @property
    def ABD(self):
        '''[ABD] Matrices as numpy.ndarray'''
        self._ABD = np.vstack([
              np.hstack([self.A, self.B]),
              np.hstack([self.B, self.D])
            ])
        return self._ABD
   
    @property
    def xiA(self):
        '''Lamination parameter xiA'''
        xiA = np.zeros(4)
        T = sum([ply.thickness for ply in self.plies])        
        for i, angle in enumerate(self.stacking):
            angle *= np.pi / 180
            zk1 = self.z_position[i+1]
            zk0 = self.z_position[i]

            xiA[0] += (zk1-zk0) * np.cos(2*angle)
            xiA[1] += (zk1-zk0) * np.sin(2*angle)
            xiA[2] += (zk1-zk0) * np.cos(4*angle)
            xiA[3] += (zk1-zk0) * np.sin(4*angle)                        
        return xiA / T

    @property
    def xiD(self):
        '''Lamination parameter xiD'''
        xiD = np.zeros(4)
        T = sum([ply.thickness for ply in self.plies])        
        for i, angle in enumerate(self.stacking):
            angle *= np.pi / 180
            zk1 = self.z_position[i+1]
            zk0 = self.z_position[i]

            xiD[0] += (zk1**3-zk0**3) * np.cos(2*angle)
            xiD[1] += (zk1**3-zk0**3) * np.sin(2*angle)
            xiD[2] += (zk1**3-zk0**3) * np.cos(4*angle)
            xiD[3] += (zk1**3-zk0**3) * np.sin(4*angle)                        
        return 4 * xiD / T**3

       #Representation
    def __repr__(self):
                    
        representation = f'composipy.LaminateProperty\n'
        representation += f'stacking = {self.stacking}'
        return representation

#Comparisons
    def __eq__(self, other):
        if isinstance(other, LaminateProperty):
            return (self.layup == other.layup)
        return NotImplemented




import numbers


class ComposipyValidator:

    def _float_number(self, value, n_min=None, n_max=None, name=None):
        try:
            value = float(value)
        except:
            raise ValueError(f'{name} Must be a real number')
        if n_min is not None and value < n_min:
            raise ValueError(f'{name} must be at least {n_min}')
        if n_max is not None and value > n_max:
            raise ValueError(f'{name} cannot exceed {n_max}')
        return value


    def _int_number(self, value, n_min=None, n_max=None, name=None):
        try:
            value = float(value)
        except:
            raise ValueError(f'{name} Must be a integer number')        
        if not float.is_integer(value):
            raise ValueError(f'{name} Must be a integer number')        
        if n_min is not None and value < n_min:
            raise ValueError(f'{name} must be at least {n_min}')
        if n_max is not None and value > n_max:
            raise ValueError(f'{name} cannot exceed {n_max}')
        return int(value)
    

    def _is_instance(self, value, typeobj, name=None):
        if not isinstance(value, typeobj):
            raise ValueError (f'{name} must be an instance of {typeobj}')
        return value


import itertools
import numpy as np

from composipy.utils import ComposipyValidator


__all__ = ['OrthotropicMaterial', 'IsotropicMaterial']


class Material(ComposipyValidator):
    pass


class OrthotropicMaterial(Material):
    """
    Creates an OrthotropicMaterial object.


    Parameters
    ----------
    e1 : float, int
        Young modulus in the 1st direction
    e2 : float, int
        Young modulus in the 2nd direction
    v12 : float
        poisson modulus 12
    g12 : float, int
        Shear modulus
    thickness : float, int
        thickness of the ply
    t1 : float, optional, default None
        tension allowable in direction 1.
    c1 : float, optional, default None
        compression allowable in direction 1.
    t2 : float, optional, default None
        tension allowable in direction 2.
    c2 : float, optional, default None
        compression allowable in direction 2.
    s : float, optional, default None
        shear allowable.
    name : str, optional, default None
        Name of the ply
 
    Examples
    --------
    >>> from composipy import OrthotropicMaterial
    >>> ply_1 = OrthotropicMaterial(129500, 9370, 0.38, 5240, 0.2)
    >>> ply_1.Q_0 # get the compliance matrix of the lamina
    Out:
    array([[130867.31382151,   3598.19426713,      0.     ],
        [  3598.19426713,   9468.93228191,      0.        ],
        [     0.        ,      0.        ,   5240.        ]])
 

    """

    _ids = itertools.count(1) # counts number of instances in order to generate names

    def __init__(
            self, 
            e1, 
            e2, 
            v12, 
            g12, 
            thickness, 
            t1=0, 
            c1=0, 
            t2=0, 
            c2=0, 
            s=0, 
            name=None):
        self.__e1 = self._float_number(e1, n_min=0, name='e1')
        self.__e2 = self._float_number(e2, n_min=0, name='')
        self.__v12 = self._float_number(v12, n_min=-0.5, n_max=0.5, name='v12')
        self.__g12 = self._float_number(g12, n_min=0, name='g12')
        self.__thickness = self._float_number(thickness, n_min=0, name='thickness')
        self.__t1 = self._float_number(t1, n_min=0, name='t1')
        self.__c1 = self._float_number(c1, n_min=0, name='c1')
        self.__t2 = self._float_number(t2, n_min=0, name='t2')
        self.__c2 = self._float_number(c2, n_min=0, name='c2')
        self.__s = self._float_number(s, n_min=0, name='s')
        self.__v21 = self.v12 * (self.e2/self.e1)
        self.__Q_0 = None #Calculated property
        self._id = next(self._ids)
        self.__name = name

    #Propertie getters    
    @property
    def e1(self):
        return self.__e1
    @property
    def e2(self):
        return self.__e2
    @property
    def v12(self):
        return self.__v12
    @property
    def g12(self):
        return self.__g12
    @property
    def thickness(self):
        return self.__thickness
    @property 
    def t1(self):
        return self.__t1
    @property 
    def c1(self):
        return self.__c1
    @property 
    def t2(self):
        return self.__t2
    @property 
    def c2(self):
        return self.__c2
    @property
    def s(self):
        return self.__s
    @property
    def v21(self):
        return self.__v21
    @property
    def name(self):
        if self.__name is None:
            self.__name = f'ply_{self._id}'
            return self.__name
        else:
            return self.__name

    @property
    def Q_0(self):
        """
        Get the compliance matrix of the material.       
        
        Returns
        -------
        self.Q_0 : numpy.ndarray
            Compliance matrix (3x3) of the Material 
        """

        if self.__Q_0 is None:
            Q11 = self.e1 / (1-self.v12*self.v21) 
            Q12 = (self.v12*self.e2) / (1-self.v12*self.v21)
            Q22 = self.e2 / (1-self.v12*self.v21)
            Q66 = self.g12
            self.__Q_0 = np.array([[Q11, Q12, 0],
                                   [Q12, Q22, 0],
                                   [0, 0, Q66]])
        return self.__Q_0

    def __repr__(self):
        return (
            f'Ply({self.name}, E1 = {self.e1}, E2 = {self.e2}, \n\
            v12 = {self.v12}, G12 = {self.g12}, thickness = {self.thickness}, \n\
            T1 = {self.t1}, C1 = {self.c1}, T2 = {self.t2}, C2 = {self.c2}, S = {self.s})')
    
#Comparisons
    def __eq__(self, other):
        if isinstance(other, OrthotropicMaterial):
            return (self.e1 == other.e1
                    and self.e2 == other.e2
                    and self.v12 == other.v12
                    and self.g12 == other.g12
                    and self.thickness == other.thickness
                    and self.t1 == other.t1
                    and self.c1 == other.c1
                    and self.t2 == other.t2
                    and self.c2 == other.c2
                    and self.s == other.s)
        else:
            raise ValueError(f'Not a instance of Material')


class IsotropicMaterial(Material):
    """Creates an instance of Ply.
    Parameters
    ----------
    e : float
        Young modulus
    v : float
        poisson modulus 12
    g : float
        Shear modulus
    thickness : float
        thickness of the ply
    t : float, default None
        tension allowable.
    c : float, default None
        compression allowable.
    s : float, default None
        shear allowable.
    name : str, default None
        Name of the ply
    """

    _ids = itertools.count(1) # counts number of instances in order to generate names

    def __init__(
            self, 
            e, 
            v, 
            g, 
            thickness, 
            t=None, 
            c=None, 
            s=None, 
            name=None):
        self.__e = self._float_number(e, n_min=0, name='e')
        self.__v = self._float_number(v, n_min=-0.5, n_max=0.5, name='v')
        self.__g = self._float_number(g, n_min=0, name='g')
        self.__thickness = self._float_number(thickness, n_min=0, name='thickness')
        self.__t = self._float_number(t, n_min=0, name='t')
        self.__c = self._float_number(c, n_min=0, name='c')
        self.__s = self._float_number(s, n_min=0, name='s')
        self.__Q_0 = None #Calculated property
        self._id = next(self._ids)
        self.__name = name

    #Propertie getters    
    @property
    def e(self):
        return self.__e

    @property
    def v(self):
        return self.__v
    @property
    def g(self):
        return self.__g
    @property
    def thickness(self):
        return self.__thickness
    @property 
    def t(self):
        return self.__t
    @property 
    def c(self):
        return self.__c
    @property
    def s(self):
        return self.__s
    @property
    def v(self):
        return self.__v
    @property
    def name(self):
        if self.__name is None:
            self.__name = f'ply_{self._id}'
            return self.__name
        else:
            return self.__name

    @property
    def Q_0(self):
        """
        Get the compliance matrix of the material.       
        
        Returns
        -------
        self.Q_0 : numpy.ndarray
            Compliance matrix (3x3) of the Material 
        """

        if self.__Q_0 is None:
            Q11 = self.e / (1-self.v*self.v) 
            Q12 = (self.v*self.e) / (1-self.v*self.v)
            Q22 = self.e / (1-self.v*self.v)
            Q66 = self.g
            self.__Q_0 = np.array([[Q11, Q12, 0],
                                   [Q12, Q22, 0],
                                   [0, 0, Q66]])
        return self.__Q_0

    def __repr__(self):
        return (
            f'IsotropicMaterial({self.name}, E = {self.e}, \n\
            v = {self.v}, G = {self.g}, thickness = {self.thickness}, \n\
            T = {self.t}, C = {self.c}, S = {self.s})')

#Comparisons
    def __eq__(self, other):
        if isinstance(other, IsotropicMaterial):
            return (self.e == other.e
                    and self.v == other.v
                    and self.g == other.g
                    and self.thickness == other.thickness
                    and self.t == other.t
                    and self.c == other.c
                    and self.s == other.s)
        else:
            raise ValueError(f'Not a instance of Material')


import sys

from composipy.pre_integrated_component._ii_F import *
from composipy.pre_integrated_component._S import *


__all__ = ['ii_ff', 'ii_fxi_f', 'ii_fxi_fxi', 'ii_fxixi_f', 'ii_fxixi_fxi', 'ii_fxixi_fxixi']
__all__.extend(['ii_f_fxi', 'ii_f_fxixi', 'ii_fxi_fxixi'])
__all__.extend(['fxi', 'sxieta'])


def ii_ff(ik):
    return eval(ii_FF[ik])


def ii_fxi_f(ik):
    return eval(ii_FXI_F[ik])


def ii_f_fxi(ik):
    ki = ik[::-1]
    return eval(ii_FXI_F[ki])


def ii_fxi_fxi(ik):
    return eval(ii_FXI_FXI[ik])


def ii_fxixi_f(ik):
    return eval(ii_FXIXI_F[ik])


def ii_f_fxixi(ik):
    ki = ik[::-1]
    return eval(ii_FXIXI_F[ki])


def ii_fxixi_fxi(ik):
    return eval(ii_FXIXI_FXI[ik])


def ii_fxi_fxixi(ik):
    ki = ik[::-1]
    return eval(ii_FXIXI_FXI[ki])


def ii_fxixi_fxixi(ik):
    return eval(ii_FXIXI_FXIXI[ik])
    

def fxi(i, xi):
    return eval(FXI[i])
    

def sxieta(ij, xi, eta):
    return eval(S[ij])


import os
import sys

from .version import __version__
from .core.material import OrthotropicMaterial, IsotropicMaterial
from .core.property import LaminateProperty
from .core.structure import PlateStructure


import sympy as sp
import numpy as np
from time import time

ti = time()

xi, eta = sp.symbols(['xi', 'eta'])
f1 = 1/2 - 3/4*xi + 1/4*xi**3
f2 = 1/8 - 1/8*xi - 1/8*xi**2 + 1/8*xi**3
f3 = 1/2 + 3/4*xi - 1/4*xi**3
f4 = -1/8 - 1/8*xi + 1/8*xi**2 + 1/8*xi**3

g1 = 1/2 - 3/4*eta + 1/4*eta**3
g2 = 1/8 - 1/8*eta - 1/8*eta**2 + 1/8*eta**3
g3 = 1/2 + 3/4*eta - 1/4*eta**3
g4 = -1/8 - 1/8*eta + 1/8*eta**2 + 1/8*eta**3

FXI = {0: f1, 1: f2, 2: f3, 3: f4} # first terms of set of functions
GETA = {0: g1, 1: g2, 2: g3, 3: g4}


def _orth_polynomials(r, sym):   
    '''
    r term from the set of functions used by Bardell (1991)

    Parameters
    ----------
    r : int
    sym : sympy.symbol
        symbol to be used in equation

    Returns
    -------
    f_sym_r : sympy equation
    r term from the set of functions.
    '''
    nmax = r//2 + 1 #denotes its own integer part (Bardel, 1991).
    f_sym_r = 0

    for n in range(0, nmax):
        f_sym_r += (
            (-1)**n * sp.factorial2(2*r-2*n-7)
            / (2**n * sp.factorial(n) * sp.factorial(r-2*n-1))
            * sym**(r-2*n-1)
            )
    f_sym_r = sp.sympify(f_sym_r)
    return f_sym_r


def convert_to_str(d, name):
    '''
    Parameters
    ----------
    d : dict 
        dictionary to be converted into string
    name : str
        name of the variable
    '''

    txt = name + ' = {\n'
    for k, v in d.items():
        txt += '    ' + str(k) + ': \'' + str(v) + '\',\n'
    txt += '}' + '\n' * 3
    return txt


if __name__ == '__main__':
    
    n_terms = 50

    # Complete set of bardell functions
    for i in range(4, n_terms):
        FXI[i] = _orth_polynomials(i+1, xi)
        GETA[i] = _orth_polynomials(i+1, eta)

    S = {} # shape function
    SXI, SETA, SXIXI, SETAETA, SXIETA = {}, {}, {}, {}, {} # derivatives


    for i, fi in FXI.items():
        for j, gj in GETA.items():
            S[(i, j)] = fi * gj 
                

    txt = f'# This file is generated by {__file__}, it contains shape function \n\n'
    txt += '__all__ = [\'FXI\', \'S\']\n\n\n'
    txt += convert_to_str(FXI, 'FXI')
    txt += convert_to_str(S, 'S')
   

    with open('_S.py', 'w') as f:
        f.write(txt)
        
    print(f'time is {time() - ti} s')




import setuptools
import sys
import os

sys.path.insert(0, ("./composipy"))

from version import __version__


with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

with open("requirements.txt", "r", encoding="utf-8") as f:
    requires = f.readlines()
    requires = [r.replace('\n', '') for r in requires]


setuptools.setup(
    name="composipy",
    version=__version__,
    install_requires=requires,
    author="Rafael Pereira",
    author_email="rafaelpsilva07@gmail.com",
    description="This package intends to perform composite material calculations",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rafaelpsilva07/composipy",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)